# -*- coding: utf-8 -*-
#
# univention-printer-assignment
#  listener module
#
# Copyright 2013 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

__package__=''  # workaround for PEP 366
import listener
import univention.debug
import univention.uldap

import tempfile
import os
import shutil

name='univention-printer-assignment'
description='create windows netlogon scripts for univention-printer-assignment'
filter='(objectClass=univentionGroup)'
attributes=[]

# globales
windowsHosts = {}
printers = {}
lo = None
sambaPaths = [
	"/var/lib/samba/netlogon/univention-printer-assignment/",
	"/var/lib/samba/sysvol/%s/scripts/univention-printer-assignment/" 
	% listener.configRegistry.get("kerberos/realm", "").lower()
]

def getConnection():

	global lo

	if not lo:
		listener.setuid(0)
		try:
			lo = univention.uldap.getMachineConnection(ldap_master=False)
		finally:
			listener.unsetuid()

def isWindowsComputer(host):

	global lo
	global windowsHosts

	getConnection()

	if windowsHosts.get(host):
		return True

	try:
		lfilter = "(&(uid=%s)(objectclass=univentionWindows))" % host
		res = lo.search(filter=lfilter, attr=["cn"])
		if res:
			windowsHosts[host] = 1
			return True
	except:
		pass

	return False

def getPrinter(dns):

	global lo
	global printers

	getConnection()

	for dn in dns:
		if not printers.get(dn):
			try:
				printers[dn] = {}
				printers[dn]["spooler"] = lo.getAttr(dn, "univentionPrinterSpoolHost")
				printers[dn]["name"] = lo.getAttr(dn, "cn")
			except:
				pass

def writePrinterList(fileName, data):

	univention.debug.debug(
		univention.debug.LISTENER,
		univention.debug.INFO,
		"univention-printer-assignment: writePrinterList host:%s data:%s" % (os.path.basename(fileName), data))

	# create temp. printer list
	(fdtemp, fdname) = tempfile.mkstemp()
	fd = os.fdopen(fdtemp, 'w')
	for i in data:
		fd.write("%s\n" % i)
	fd.close()
	
	# copy temp. printer list to netlogon dir
	listener.setuid(0)
	try:
		shutil.move(fdname, fileName)
		os.chmod(fileName, 0644)
		os.chown(fileName, 0, 0)
	except:
		univention.debug.debug(
			univention.debug.LISTENER,
			univention.debug.ERROR,
			 "univention-printer-assignment: failed to create printer list %s" % fileName)
	finally:
		listener.unsetuid()

	return 0


def deleteFromPrinterList(groupDn, fileName):

	univention.debug.debug(
		univention.debug.LISTENER,
		univention.debug.INFO,
		"univention-printer-assignment: deleteFromPrinterList group:%s host:%s" % (groupDn, os.path.basename(fileName)))

	# read printer list
	fh = None
	try:
		fh = open(fileName, "r")
	except IOError:
		pass

	entries = []
	newEntries = []

	if fh:
		entries = fh.readlines()
		fh.close()

	# delete entries ending with ,dn (of the group)
	for i in entries:
		i = i.strip()
		item = i.split(":")
		if item and item[-1] != groupDn:
			newEntries.append(i)

	# write printer list
	writePrinterList(fileName, newEntries)

	return newEntries


def addToPrinterList(groupDn, samba3File, printersDn):

	global printers

	univention.debug.debug(
		univention.debug.LISTENER,
		univention.debug.INFO,
		"univention-printer-assignment: modifyPrinterList group:%s host:%s printers:%s" % 
		(groupDn, os.path.basename(samba3File), printersDn))

	# get spool host for printers -> global printers
	getPrinter(printersDn)

	# delete group entries from printer list
	newEntries = deleteFromPrinterList(groupDn, samba3File)

	# add entries for this group
	for dn in printersDn:
		prt = printers.get(dn)
		if prt:
			spoolHosts = prt.get("spooler")
			name = prt.get("name")
			if spoolHosts and name:
				# printername, spoolhost1 (, spoolhost2, ...), groupdn to identify entry
				entry = "%s:%s:%s" % (name[0], ",".join(spoolHosts), groupDn)
				newEntries.append(entry)

	# write new printer list
	writePrinterList(samba3File, newEntries)

def checkObjectForWindowsHostsAndPrinter(obj):

	hosts = []

	# is there a printer addignment
	printerDn = obj.get("univentionAssignedPrinter", [])
	if printerDn:
		# is there a windows computer as member
		for member in obj.get("memberUid", []):
			if member.endswith("$") and isWindowsComputer(member):
				hosts.append(member.replace("$", "").upper())

	return hosts, printerDn

def syncToSamba4(fileName, host):

	global sambaPaths

	samba4File = os.path.join(sambaPaths[1], "%s.printerlist" % host)

	listener.setuid(0)
	try:
		shutil.copyfile(fileName, samba4File)
	except:
		univention.debug.debug(
			univention.debug.LISTENER,
			univention.debug.ERROR,
			"univention-printer-assignment: failed to copy s3 %s to s4 %s" % (fileName, samba4File))
	finally:
		listener.setuid(0)


def handler(dn, new, old):

	global sambaPaths

	# create samba paths
	for path in sambaPaths:
		listener.setuid(0)
		try:
			if not os.path.isdir(path):
				os.makedirs(path)
		except:
			univention.debug.debug(
				univention.debug.LISTENER,
				univention.debug.ERROR,
				"univention-printer-assignment: failed to create dir %s" % path)
		finally:
			listener.unsetuid()

	if old:
		hosts, printerDn = checkObjectForWindowsHostsAndPrinter(old)
		if printerDn:
			for host in hosts:
				samba3File = os.path.join(sambaPaths[0], "%s.printerlist" % host)
				univention.debug.debug(
					univention.debug.LISTENER,
					univention.debug.INFO,
					"%s: delete group:%s host:%s" % (name, dn, host))
				# do it
				deleteFromPrinterList(dn, samba3File)
				syncToSamba4(samba3File, host)

	if new:
		hosts, printerDn = checkObjectForWindowsHostsAndPrinter(new)
		if printerDn:
			for host in hosts:
				samba3File = os.path.join(sambaPaths[0], "%s.printerlist" % host)
				univention.debug.debug(
					univention.debug.LISTENER,
					univention.debug.INFO,
					"%s: create/modify group:%s host:%s" % (name, dn, host))
				# do it
				addToPrinterList(dn, samba3File, printerDn)
				syncToSamba4(samba3File, host)

def clean():
	pass

def postrun():

	global windowsHosts
	global printers
	global lo

	hosts = {}
	printers = {}
	lo = None

def initialize():
	pass

